/*

Overall goal: 	Delay Routines

Designed by Shane Tolmie Sept 2001

Microprocessor: 12Cx67x

Compiled with:	Hi-Tech C v7.87pl2

*/

#include	"always.h"

#if (PIC_CLK == 4)
	#define dly125n please remove; for 32Mhz+ only
	#define dly250n please remove; for 16Mhz+ only
	#define dly500n please remove; for 8Mhz+ only
	#define dly1u asm("nop")
	#define dly2u dly1u;dly1u
#elif (PIC_CLK == 8)
	#define dly125n please remove; for 32Mhz+ only
	#define dly250n please remove; for 16Mhz+ only
	#define dly500n asm("nop")
	#define dly1u dly500n;dly500n
	#define dly2u dly1u;dly1u
#elif (PIC_CLK == 16)
	#define dly125n please remove; for 32Mhz+ only
	#define dly250n asm("nop")
	#define dly500n dly250n;dly250n
	#define dly1u dly500n;dly500n
	#define dly2u dly1u;dly1u
#elif (PIC_CLK == 20)
	#define dly200n asm("nop")
	#define dly400n dly250n;dly250n
	#define dly2u dly400n;dly400n;dly400n;dly400n;dly400n
#elif (PIC_CLK == 32)
	#define dly125n asm("nop")
	#define dly250n dly125n;dly125n
	#define dly500n dly250n;dly250n
	#define dly1u dly500n;dly500n
	#define dly2u dly1u;dly1u
#elif
	#error Define pic_clk for dly???n in delay.c
#endif

//delay routine

#if PIC_CLK == 4
	#define DelayDivisor 4
	#define WaitUntil_1Us asm("nop")
	#define Jumpback asm("goto $ - 2")
#elif PIC_CLK == 8
	#define DelayDivisor 2
	#define WaitUntil_1Us asm("nop")
	#define Jumpback asm("goto $ - 2")
#elif PIC_CLK == 16
	#define DelayDivisor 1
	#define WaitUntil_1Us asm("nop")
	#define Jumpback asm("goto $ - 2")
#elif PIC_CLK == 32
	#define DelayDivisor 1
	#define WaitUntil_1Us asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop")
	#define Jumpback asm("goto $ - 6")
#elif
	#error Please define pic_clk for DelayUs(?) in delay.c
#endif

unsigned char delayus_variable;

#define DelayUs(x) \
	delayus_variable=(unsigned char)((x)/DelayDivisor); \
	WaitUntil_1Us; \
	asm("decfsz _delayus_variable,f"); \
	Jumpback;

/*

timeouts:

C code for testing with ints:

			unsigned int timeout;
			timeout=4000;
			PORT_DIRECTION=OUTPUT;
			while(1)
			{
				PORT=1;
				timeout=8000;
				while(timeout-- >= 1);	//60ms @ 8Mhz, opt on, 72ms @ 8Mhz, opt off
				PORT=0;
			}

Time taken:	optimisations on:		16cyc/number loop, 8us @ 8Mhz
			optimisations off:		18cyc/number loop, 9us @ 8Mhz
			with extra check ie:	&& (RB7==1), +3cyc/number loop, +1.5us @ 8Mhz

C code for testing with chars:

			similar to above

Time taken:	optimisations on:		9cyc/number loop, 4.5us @ 8Mhz
			with extra check ie:	&& (RB7==1), +3cyc/number loop, +1.5us @ 8Mhz

Formula:	rough timeout value = (<us desired>/<cycles per loop>) * (PIC_CLK/4.0)

To use:		//for max  timeout of 1147us @ 8Mhz
			#define LOOP_CYCLES_CHAR	9					//how many cycles per loop, optimizations on
			#define timeout_char_us(x)	(unsigned char)((x/LOOP_CYCLES_CHAR)*(PIC_CLK/4.0))
			unsigned char timeout;
			timeout=timeout_char_us(1147);						//max timeout allowed @ 8Mhz, 573us @ 16Mhz
			while((timeout-- >= 1) && (<extra condition>));	//wait

To use:		//for max 491512us, half sec timeout @ 8Mhz
			#define LOOP_CYCLES_INT		16					//how many cycles per loop, optimizations on
			#define timeout_int_us(x)	(unsigned int)((x/LOOP_CYCLES_INT)*(PIC_CLK/4.0))
			unsigned int timeout;
			timeout=timeout_int_us(491512);						//max timeout allowed @ 8Mhz
			while((timeout-- >= 1) && (<extra condition>));	//wait
*/
#define LOOP_CYCLES_CHAR	9							//how many cycles per loop, optimizations on
#define timeout_char_us(x)	(((x)/LOOP_CYCLES_CHAR)*(PIC_CLK/4))

#define LOOP_CYCLES_INT		16							//how many cycles per loop, optimizations on
#define timeout_int_us(x)	(((x)/LOOP_CYCLES_INT)*(PIC_CLK/4))

//*****
//large delay functions
/*
void DelayBigUs(unsigned int cnt)
{
	unsigned char i;

	i=(unsigned char)(cnt>>8);
	while (i>=1)
	{
		i--;
		DelayUs(253);
		CLRWDT();
	}
	DelayUs((unsigned char)(cnt & 0xFF));
}
*/
/*
void DelayBigUs_interrupt(unsigned int cnt)
{
	unsigned char i;

	i=(unsigned char)(cnt>>8);
	while (i>=1)
	{
		i--;
		DelayUs(253);
		//CLRWDT();
	}
	DelayUs((unsigned char)(cnt & 0xFF));
}
*/
/*
void DelayMs(unsigned char cnt)
{
	unsigned char i;
	do
	{
		i=4;
		do
		{
			DelayUs(250);
		} while (--i);
	} while (--cnt);
}
*/
/*
//this copy is for the interrupt function
void DelayMs_interrupt(unsigned char cnt)
{
	unsigned char	i;
	do {
		i = 4;
		do {
			DelayUs(250);
		} while(--i);
	} while(--cnt);
}
*/
/*
void DelayBigMs(unsigned int cnt)
{
	unsigned char i;
	do
	{
		i=4;
		do
		{
			DelayUs(250);
			CLRWDT();
		} while (--i);
	} while (--cnt);
}
*/
/*
void DelayS(unsigned char cnt)
{
	unsigned char i;
	do
	{
		i=4;
		do
		{
			DelayMs(250);
			//CLRWDT();
		} while (--i);
	} while (--cnt);
}
*/
